package com.flyfish.framework.r2dbc.metadata.reference;

import com.flyfish.framework.annotations.Property;
import lombok.AllArgsConstructor;
import lombok.Data;

import java.lang.reflect.Method;

@Data
@Property("一对一关联查询")
@AllArgsConstructor
public class R2dbcAssociation {

    @Property("关联实体类")
    private Class<?> entityClass;

    @Property("关联字段")
    private String field;

    @Property("目标字段")
    private org.springframework.data.mapping.model.Property targetField;

    @Property("字段是否在当前实体内部")
    boolean inner;

    public static R2dbcAssociation of(org.springframework.data.mapping.model.Property targetField, String field, boolean inner) {
        return new R2dbcAssociation(targetField.getType(), field, targetField, inner);
    }

    public boolean isValid() {
        return targetField.hasAccessor();
    }

    public <T> T setValue(T obj, Object value) {
        if (null != value && targetField.hasAccessor()) {
            targetField.getSetter().ifPresent(method -> invokeSetter(method, obj, value));
        }
        return obj;
    }

    private void invokeSetter(Method method, Object obj, Object value) {
        try {
            method.invoke(obj, value);
        } catch (Exception ignored) {

        }
    }
}
